var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __esm = (fn, res) => function __init() {
  return fn && (res = (0, fn[__getOwnPropNames(fn)[0]])(fn = 0)), res;
};
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/utils.ts
import { AlignmentType, HeadingLevel } from "docx";
function getHeadingLevel(level) {
  if (level == null) {
    return void 0;
  }
  switch (level) {
    case 0:
      return HeadingLevel.TITLE;
    case 1:
      return HeadingLevel.HEADING_1;
    case 2:
      return HeadingLevel.HEADING_2;
    case 3:
      return HeadingLevel.HEADING_3;
    case 4:
      return HeadingLevel.HEADING_4;
    case 5:
      return HeadingLevel.HEADING_5;
    case 6:
      return HeadingLevel.HEADING_6;
    default:
      return HeadingLevel.HEADING_6;
  }
}
function getTextAlignment(align) {
  switch (align) {
    case "left":
      return AlignmentType.LEFT;
    case "center":
      return AlignmentType.CENTER;
    case "right":
      return AlignmentType.RIGHT;
    default:
      return void 0;
  }
}
function getImageTokens(tokenList, tokens = []) {
  for (const token of tokenList) {
    if (!token) continue;
    if (token.type === "image") {
      tokens.push(token);
    } else if (token.tokens?.length) {
      getImageTokens(token.tokens, tokens);
    }
  }
  return tokens;
}
function getImageExtension(filename = "", mime) {
  let ext = "";
  switch (mime) {
    case "image/jpeg":
      ext = "jpg";
      break;
    case "image/png":
      ext = "png";
      break;
    case "image/gif":
      ext = "gif";
      break;
    case "image/bmp":
      ext = "bmp";
      break;
    case "image/webp":
      ext = "webp";
      break;
    case "image/svg+xml":
      ext = "svg";
      break;
    default:
      const name = filename.split("?").pop() || "";
      const index = name.lastIndexOf(".");
      if (index > -1) {
        ext = name.substring(index + 1);
      }
      break;
  }
  if (!ext) {
    throw new Error(`Cannot get Image extension from mime type: ${mime}`);
  } else if (!ImageTypeWhitelist.has(ext)) {
    throw new Error(`Image extension ${ext} is not supported`);
  }
  return ext;
}
function isHttp(src) {
  return /^https?:\/\//.test(src);
}
var ImageTypeWhitelist;
var init_utils = __esm({
  "src/utils.ts"() {
    "use strict";
    ImageTypeWhitelist = /* @__PURE__ */ new Set(["jpg", "png", "gif", "bmp", "webp"]);
  }
});

// src/adapters/nodejs.ts
var nodejs_exports = {};
__export(nodejs_exports, {
  downloadImage: () => downloadImage
});
import imagesize, { disableTypes } from "image-size";
import fs from "node:fs/promises";
import http from "node:http";
import https from "node:https";
function loadImage(src) {
  if (isHttp(src)) {
    return new Promise((resolve, reject) => {
      const agent = src.startsWith("https") ? https : http;
      agent.get(src, (res) => {
        const chunks = [];
        res.on("data", (chunk) => {
          chunks.push(chunk);
        });
        res.on("end", () => {
          const buffer = Buffer.concat(chunks);
          disableTypes(["svg", "webp"]);
          resolve(buffer);
        });
        res.on("error", (err) => {
          reject(new Error(`Failed to load image: ${err.message || err}`));
        });
      });
    });
  }
  return fs.readFile(src);
}
var downloadImage;
var init_nodejs = __esm({
  "src/adapters/nodejs.ts"() {
    "use strict";
    init_utils();
    downloadImage = async function(token) {
      const src = token.href;
      if (!src) {
        return null;
      }
      try {
        const buffer = await loadImage(src);
        const { width, height, type } = imagesize(buffer);
        const supportType = getImageExtension(src, type);
        if (!supportType) {
          return null;
        }
        if (supportType === "webp") {
          console.error(`[MarkdownDocx] Webp is not supported in the nodejs environment`);
          return null;
        }
        return {
          type: supportType,
          data: buffer,
          width,
          height
        };
      } catch (error) {
        console.error(`[MarkdownDocx] downloadImageError`, error);
        return null;
      }
    };
  }
});

// src/index.ts
import { Packer } from "docx";

// src/MarkdownDocx.ts
import { Document } from "docx";

// src/renders/render-blocks.ts
import { Paragraph as Paragraph2 } from "docx";

// src/styles/classes.ts
var classes = {
  // Paragraph styles
  Space: "MdSpace",
  Code: "MdCode",
  Hr: "MdHr",
  Blockquote: "MdBlockquote",
  Html: "MdHtml",
  Def: "MdDef",
  Paragraph: "MdParagraph",
  Text: "MdText",
  Footnote: "MdFootnote",
  ListItem: "MdListItem",
  Table: "MdTable",
  TableHeader: "MdTableHeader",
  TableCell: "MdTableCell",
  Heading1: "MdHeading1",
  Heading2: "MdHeading2",
  Heading3: "MdHeading3",
  Heading4: "MdHeading4",
  Heading5: "MdHeading5",
  Heading6: "MdHeading6",
  // Character styles
  Tag: "MdTag",
  Link: "MdLink",
  Strong: "MdStrong",
  Em: "MdEm",
  Codespan: "MdCodespan",
  Del: "MdDel",
  Br: "MdBr"
};

// src/styles/colors.ts
var colors = {
  primary: "4472C4",
  // Blue
  secondary: "70AD47",
  // Green
  accent1: "ED7D31",
  // Orange
  accent2: "A5A5A5",
  // Gray
  accent3: "FFC000",
  // Yellow
  heading1: "2F5597",
  // Dark Blue
  heading2: "5B9BD5",
  // Medium Blue
  heading3: "44546A",
  // Dark Gray
  link: "0563C1",
  // Bright Blue
  code: "C7254E",
  // Red
  blockquote: "666666",
  // Medium Gray
  del: "FF0000"
  // Red
};

// src/styles/markdown.ts
import { UnderlineType } from "docx";
var markdown = {
  space: {
    className: classes.Space,
    run: {
      size: 12
      // 6pt - small space
    },
    paragraph: {
      spacing: {
        before: 0,
        after: 0
      }
    }
  },
  code: {
    className: classes.Code,
    run: {
      font: "Courier New",
      size: 22,
      // 11pt
      color: colors.secondary
    },
    paragraph: {
      border: {
        top: {
          style: "single",
          size: 1,
          color: colors.accent2,
          space: 8
        },
        bottom: {
          style: "single",
          size: 1,
          color: colors.accent2,
          space: 8
        },
        left: {
          style: "single",
          size: 1,
          color: colors.accent2,
          space: 8
        },
        right: {
          style: "single",
          size: 1,
          color: colors.accent2,
          space: 8
        }
      },
      spacing: {
        before: 200,
        after: 200
      }
    }
  },
  hr: {
    className: classes.Hr,
    paragraph: {
      border: {
        bottom: {
          style: "single",
          size: 1,
          color: colors.accent2,
          space: 1
        }
      },
      spacing: {
        before: 240,
        after: 240
      }
    }
  },
  blockquote: {
    className: classes.Blockquote,
    run: {
      color: colors.blockquote,
      italics: true
    },
    paragraph: {
      border: {
        left: {
          style: "single",
          size: 20,
          color: colors.accent2,
          space: 12
        }
      },
      indent: {
        left: 360
        // 0.25 inch
      },
      spacing: {
        before: 200,
        after: 200
      }
    }
  },
  html: {
    className: classes.Html,
    run: {
      font: "Courier New",
      color: colors.accent1
    }
  },
  def: {
    className: classes.Def,
    paragraph: {
      indent: {
        left: 720,
        // 0.5 inch
        hanging: 360
        // 0.25 inch
      }
    }
  },
  paragraph: {
    className: classes.Paragraph,
    paragraph: {
      spacing: {
        before: 120,
        after: 120
      }
    }
  },
  text: {
    className: classes.Text
  },
  footnote: {
    className: classes.Footnote,
    run: {
      superScript: true
    }
  },
  listItem: {
    className: classes.ListItem,
    paragraph: {
      indent: {
        left: 720,
        // 0.5 inch
        hanging: 360
        // 0.25 inch
      },
      spacing: {
        before: 60,
        after: 60
      }
    }
  },
  table: {
    className: classes.Table,
    paragraph: {
      spacing: {
        before: 60,
        after: 60
      }
    }
  },
  tableHeader: {
    className: classes.TableHeader,
    // special
    properties: {
      shading: {
        fill: "F1F2F1"
      }
    },
    run: {
      bold: true
    }
  },
  tableCell: {
    className: classes.TableCell
  },
  heading1: {
    className: classes.Heading1,
    run: {
      size: 36,
      // 18pt
      bold: true,
      color: colors.heading1
    },
    paragraph: {
      spacing: {
        before: 480,
        // 24pt
        after: 240
        // 12pt
      },
      keepNext: true,
      outlineLevel: 0
    }
  },
  heading2: {
    className: classes.Heading2,
    run: {
      size: 32,
      // 16pt
      bold: true,
      color: colors.heading2
    },
    paragraph: {
      spacing: {
        before: 400,
        // 20pt
        after: 200
        // 10pt
      },
      keepNext: true,
      outlineLevel: 1
    }
  },
  heading3: {
    className: classes.Heading3,
    run: {
      size: 28,
      // 14pt
      bold: true,
      color: colors.heading3
    },
    paragraph: {
      spacing: {
        before: 320,
        // 16pt
        after: 160
        // 8pt
      },
      keepNext: true,
      outlineLevel: 2
    }
  },
  heading4: {
    className: classes.Heading4,
    run: {
      size: 26,
      // 13pt
      bold: true,
      color: colors.heading3
    },
    paragraph: {
      spacing: {
        before: 280,
        // 14pt
        after: 140
        // 7pt
      },
      keepNext: true,
      outlineLevel: 3
    }
  },
  heading5: {
    className: classes.Heading5,
    run: {
      size: 24,
      // 12pt
      bold: true,
      italics: true,
      color: colors.heading3
    },
    paragraph: {
      spacing: {
        before: 240,
        // 12pt
        after: 120
        // 6pt
      },
      keepNext: true,
      outlineLevel: 4
    }
  },
  heading6: {
    className: classes.Heading6,
    run: {
      size: 24,
      // 12pt
      bold: false,
      italics: true,
      color: colors.heading3
    },
    paragraph: {
      spacing: {
        before: 240,
        // 12pt
        after: 120
        // 6pt
      },
      keepNext: true,
      outlineLevel: 5
    }
  },
  // inline
  tag: {
    inline: true,
    className: classes.Tag,
    run: {
      font: "Courier New",
      color: colors.accent1
    }
  },
  link: {
    inline: true,
    className: classes.Link,
    run: {
      color: colors.link,
      underline: {
        type: UnderlineType.SINGLE
      }
    }
  },
  strong: {
    inline: true,
    className: classes.Strong,
    run: {
      bold: true
    }
  },
  em: {
    inline: true,
    className: classes.Em,
    run: {
      italics: true
    }
  },
  codespan: {
    inline: true,
    className: classes.Codespan,
    run: {
      font: "Courier New",
      color: colors.secondary
    }
  },
  del: {
    inline: true,
    className: classes.Del,
    run: {
      strike: true,
      color: colors.del
    }
  },
  br: {
    inline: true,
    className: classes.Br
    // The actual line break will be handled in the rendering code
  }
};

// src/styles/numbering.ts
import { LevelFormat } from "docx";
var numbering = {
  config: [
    {
      reference: "numbering-points",
      levels: [
        makeNumbering(0),
        makeNumbering(1),
        makeNumbering(2),
        makeNumbering(3),
        makeNumbering(4),
        makeNumbering(5),
        makeNumbering(6),
        makeNumbering(7),
        makeNumbering(8)
      ]
    },
    {
      reference: "bullet-points",
      levels: [
        makeBullet(0, "\u2022"),
        makeBullet(1, "\u25A0"),
        makeBullet(2, "\u25B6"),
        makeBullet(3, "\u25B2"),
        makeBullet(4, "\u25C6"),
        makeBullet(5, "\u25CF"),
        makeBullet(6, "\u25A1")
      ]
    }
  ]
};
function makeNumbering(level) {
  const opt = {
    level,
    format: LevelFormat.DECIMAL,
    text: level < 1 ? "%1" : level < 2 ? "%1.%2" : level < 3 ? "%1.%2.%3" : `%${level + 1})`
  };
  return opt;
}
function makeBullet(level, charset) {
  const opt = {
    level,
    format: LevelFormat.BULLET,
    text: charset
  };
  return opt;
}

// src/styles/styles.ts
import {
  UnderlineType as UnderlineType2
} from "docx";
var defaultStyle = {
  document: {
    run: {
      size: 24
      // 12pt
    },
    paragraph: {
      spacing: { lineRule: "auto" }
    }
  },
  hyperlink: {
    run: {
      color: colors.link,
      underline: {
        type: UnderlineType2.SINGLE,
        color: colors.link
      }
    }
  },
  heading1: {},
  heading2: {},
  heading3: {},
  heading4: {},
  heading5: {},
  heading6: {},
  strong: {},
  listParagraph: {},
  footnoteReference: {},
  footnoteText: {},
  footnoteTextChar: {},
  title: {}
};
function createDocumentStyle(options) {
  const paragraphStyles = [];
  const characterStyles = [];
  const keys = Object.keys(markdown);
  for (const key of keys) {
    const style = markdown[key];
    if (!style) continue;
    const { className, run, inline, paragraph, basedOn = "Normal", next = "Normal", quickFormat = true } = style;
    if (inline) {
      characterStyles.push({ id: className, name: className, basedOn, next, quickFormat, run });
    } else {
      paragraphStyles.push({ id: className, name: className, basedOn, next, quickFormat, run, paragraph });
    }
  }
  return {
    default: defaultStyle,
    paragraphStyles,
    characterStyles,
    ...options
  };
}

// src/styles/index.ts
var styles = {
  colors,
  classes,
  default: defaultStyle,
  markdown,
  numbering
};

// src/renders/render-list.ts
function renderList(render, block, attr) {
  const list = {
    level: typeof attr.list?.level === "number" ? attr.list.level + 1 : 0,
    type: block.ordered ? "number" : "bullet"
  };
  return block.items.map((item) => {
    const tokens = item.tokens;
    const attribute = {
      ...attr,
      style: classes.ListItem,
      list: {
        ...list,
        task: item.task,
        checked: item.checked
      }
    };
    return renderBlocks(render, tokens, attribute);
  }).flat();
}

// src/renders/render-paragraph.ts
init_utils();
import { Paragraph } from "docx";

// src/renders/render-checkbox.ts
import { CheckBox } from "docx";
function renderCheckbox(render, checked) {
  return new CheckBox({
    checked: !!checked,
    checkedState: { value: "2611", font: "MS Gothic" },
    uncheckedState: { value: "2610", font: "MS Gothic" }
  });
}

// src/renders/render-text.ts
import { TextRun } from "docx";
function renderText(render, text, attr) {
  const multipleLines = text.trim().split(/\n/);
  const totalLine = multipleLines.length;
  const options = {
    style: attr.style,
    italics: attr.italics,
    bold: attr.bold,
    underline: attr.underline ? {} : void 0,
    strike: attr.strike,
    break: attr.break ? typeof attr.break === "number" ? attr.break : 1 : void 0
  };
  if (attr.strong) options.bold = true;
  if (attr.em) options.italics = true;
  if (attr.codespan) options.underline = {};
  if (attr.del) options.strike = true;
  if (totalLine > 1) {
    const textNodes = [];
    textNodes.push(...multipleLines.map((line, index) => new TextRun({
      ...options,
      text: line,
      break: index > 0 ? 1 : void 0
    })));
    return textNodes;
  }
  return [
    new TextRun({
      text,
      ...options
    })
  ];
}

// src/renders/render-tokens.ts
import { ExternalHyperlink, FootnoteReferenceRun } from "docx";

// src/renders/render-image.ts
import { ImageRun } from "docx";
function renderImage(render, block, attr) {
  if (render.ignoreImage) {
    return false;
  }
  const image = render.findImage(block);
  if (!image || !image.type) {
    return renderText(render, `[!${block.text}](${block.href})`, attr);
  }
  return new ImageRun({
    type: image.type,
    data: image.data,
    transformation: {
      width: image.width,
      height: image.height
    },
    altText: {
      title: block.title || block.text,
      description: block.text,
      name: block.text
    }
  });
}

// src/renders/render-tokens.ts
function renderTokens(render, tokens, attr = {}) {
  const children = [];
  for (const token of tokens) {
    const child = flatInlineToken(render, token, attr);
    if (Array.isArray(child)) {
      children.push(...child);
    } else if (child) {
      children.push(child);
    } else if (child == null) {
      console.warn(`Inline token is empty: ${token.type}`);
    }
  }
  return children;
}
function flatInlineToken(render, token, attr) {
  switch (token.type) {
    case "escape":
      return renderText(render, token.text, attr);
    case "html":
      if (render.ignoreHtml) {
        return false;
      }
      return renderText(render, token.text, { ...attr, html: true, style: classes.Tag });
    case "link":
      return new ExternalHyperlink({
        children: renderTokens(
          render,
          token.tokens,
          { ...attr, link: true, style: classes.Link }
        ),
        link: token.href
      });
    case "em":
      return renderTokens(
        render,
        token.tokens,
        { ...attr, em: true, style: classes.Em }
      );
    case "strong":
      return renderTokens(
        render,
        token.tokens,
        { ...attr, strong: true, style: classes.Strong }
      );
    case "codespan":
      return renderText(
        render,
        token.text,
        { ...attr, codespan: true, style: classes.Code }
      );
    case "br":
      return renderText(
        render,
        "",
        { break: 1, br: true, style: classes.Br }
      );
    case "del":
      return renderTokens(
        render,
        token.tokens,
        { ...attr, del: true, style: classes.Del }
      );
    case "text":
      if (token.tokens?.length) {
        return renderTokens(render, token.tokens, attr);
      }
      return renderText(render, token.text, attr);
    case "image":
      return renderImage(render, token, attr);
    case "footnoteRef":
      return new FootnoteReferenceRun(token.id);
    default:
      return null;
  }
}

// src/renders/render-paragraph.ts
function renderParagraph(render, tokens, attr) {
  const heading = getHeadingLevel(attr.heading);
  const alignment = getTextAlignment(attr.align);
  const bullet = attr.list?.type === "bullet" ? {
    level: Math.min(attr.list.level, 9)
  } : void 0;
  const numbering2 = attr.list?.type === "number" ? {
    level: Math.min(attr.list.level, 9),
    reference: "numbering-points"
  } : void 0;
  const options = {
    heading,
    alignment,
    bullet,
    numbering: numbering2,
    style: attr.style
  };
  const children = typeof tokens === "string" ? renderText(render, tokens, {}) : renderTokens(render, tokens, {});
  if (attr.list?.task) {
    children.unshift(renderCheckbox(render, attr.list.checked));
  }
  return new Paragraph({
    children,
    ...options
  });
}

// src/renders/render-table.ts
import { Table, TableCell, TableRow, VerticalAlign, WidthType } from "docx";
function renderTable(render, block, attrs) {
  const toProps = (token, isHeader) => {
    return {
      ...attrs,
      align: token?.align,
      style: isHeader ? classes.TableHeader : classes.TableCell
    };
  };
  const style = render.styles.markdown;
  return new Table({
    ...style.table.properties,
    style: classes.Table,
    width: {
      size: 100,
      type: WidthType.PERCENTAGE
    },
    rows: [
      // headers
      new TableRow({
        tableHeader: true,
        cantSplit: true,
        children: block.header.map((cell) => {
          return new TableCell({
            verticalAlign: VerticalAlign.CENTER,
            ...style.tableHeader.properties,
            children: [
              renderParagraph(render, cell.tokens, toProps(cell, true))
            ]
          });
        })
      }),
      // rows
      ...block.rows.map((row) => {
        return new TableRow({
          cantSplit: true,
          children: row.map((cell) => {
            return new TableCell({
              verticalAlign: VerticalAlign.CENTER,
              ...style.tableCell.properties,
              children: [
                renderParagraph(render, cell.tokens, toProps(cell))
              ]
            });
          })
        });
      })
    ]
  });
}

// src/renders/render-blocks.ts
function renderBlocks(render, blocks, attr = {}) {
  const paragraphs = [];
  for (const block of blocks) {
    const child = renderBlock(render, block, attr);
    if (Array.isArray(child)) {
      paragraphs.push(...child);
    } else if (child) {
      paragraphs.push(child);
    } else if (child == null) {
      console.warn(`Block is empty: ${block.type}`);
    }
  }
  return paragraphs;
}
function renderBlock(render, block, attr) {
  switch (block.type) {
    case "space":
      return new Paragraph2({
        text: "",
        style: classes.Space
      });
    case "code":
      return renderParagraph(render, block.text, { ...attr, code: true, style: "MdCode" });
    case "heading":
      return renderParagraph(render, block.tokens, {
        ...attr,
        heading: block.depth,
        // @ts-ignore
        style: classes[`Heading${block.depth}`]
      });
    case "hr":
      return new Paragraph2({
        text: "",
        thematicBreak: true,
        style: classes.Hr
      });
    case "blockquote":
      return renderBlocks(render, block.tokens, {
        ...attr,
        blockquote: true,
        style: classes.Blockquote
      });
    case "list":
      return renderList(render, block, attr);
    case "html":
      if (render.ignoreHtml) {
        return false;
      }
      return renderParagraph(render, block.text, {
        ...attr,
        code: true,
        style: classes.Html
      });
    case "def":
      return new Paragraph2({
        text: block.title,
        style: classes.Def
      });
    case "table":
      return renderTable(render, block, attr);
    case "paragraph":
      return renderParagraph(render, block.tokens, {
        style: classes.Paragraph,
        // can be overridden by attr
        ...attr
      });
    case "text":
      if (block.tokens?.length) {
        return renderParagraph(render, block.tokens, {
          style: classes.Text,
          // can be overridden by attr
          ...attr
        });
      }
      return renderParagraph(render, block.text, attr);
    case "footnote":
      const noteList = renderBlocks(render, block.tokens, {
        ...attr,
        style: classes.Footnote,
        footnote: true
      });
      render.addFootnote(block.id, noteList);
      return false;
    default:
      return null;
  }
}

// src/tokenize.ts
import { Lexer } from "marked";

// src/extensions/footnote.ts
function footnote(lexer) {
  let hasFootnotes = false;
  let footnoteId = 0;
  const footnotes = /* @__PURE__ */ new Map();
  return {
    name: "footnote",
    block: tokenizerBlock,
    inline: tokenizerInline
  };
  function tokenizerBlock(src) {
    const match = /^\[\^([^\]\n]+)\]:(?:[ \t]+|[\n]*?|$)([^\n]*?(?:\n|$)(?:\n*?[ ]{4,}[^\n]*)*)/.exec(src);
    if (!match) return;
    if (!hasFootnotes) {
      hasFootnotes = true;
      footnoteId = 0;
      footnotes.clear();
    }
    const [raw, label, text = ""] = match;
    let content = text.split("\n").reduce((acc, curr) => {
      return acc + "\n" + curr.replace(/^(?:[ ]{4}|[\t])/, "");
    }, "");
    const contentLastLine = content.trimEnd().split("\n").pop();
    content += // add lines after list, blockquote, codefence, and table
    contentLastLine && /^[ \t]*?[>\-*][ ]|[`]{3,}$|^[ \t]*?[|].+[|]$/.test(contentLastLine) ? "\n\n" : "";
    const token = {
      id: ++footnoteId,
      type: "footnote",
      raw,
      label,
      tokens: lexer.blockTokens(content.trim())
    };
    footnotes.set(label, token);
    return token;
  }
  function tokenizerInline(src) {
    const match = /^\[\^([^\]\n]+)\]/.exec(src);
    if (match) {
      const [raw, label] = match;
      const note = footnotes.get(label);
      if (!note) return;
      const ref = {
        id: note.id,
        type: "footnoteRef",
        raw,
        label
      };
      return ref;
    }
  }
}

// src/extensions/index.ts
function useExtensions(lexer) {
  usePlugin(lexer, footnote);
  return lexer;
}
function usePlugin(lexer, fn) {
  const plugin = fn(lexer);
  const extensions = lexer.options.extensions || (lexer.options.extensions = {});
  const blocks = extensions.block || (extensions.block = []);
  const inline = extensions.inline || (extensions.inline = []);
  if (plugin.block) {
    blocks.push(plugin.block);
  }
  if (plugin.inline) {
    inline.push(plugin.inline);
  }
}

// src/tokenize.ts
function tokenize(markdown2, options) {
  const laxer = useExtensions(new Lexer(options));
  return laxer.lex(markdown2);
}

// src/MarkdownDocx.ts
init_utils();
var _MarkdownDocx = class _MarkdownDocx {
  constructor(markdown2, options = {}) {
    this.markdown = markdown2;
    this.options = options;
    this.styles = styles;
    this._imageStore = /* @__PURE__ */ new Map();
    this.footnotes = {};
    this.options = {
      ..._MarkdownDocx.defaultOptions,
      ...options
    };
  }
  static covert(markdown2, _options = {}) {
    return new _MarkdownDocx(markdown2, _options).toDocument();
  }
  get ignoreImage() {
    return !!this.options.ignoreImage;
  }
  get ignoreFootnote() {
    return !!this.options.ignoreFootnote;
  }
  get ignoreHtml() {
    return !!this.options.ignoreHtml;
  }
  async toDocument(options) {
    this.footnotes = {};
    const section = await this.toSection();
    const doc = new Document({
      numbering,
      styles: createDocumentStyle(),
      ...this.options.document,
      ...options,
      footnotes: this.footnotes,
      sections: [
        {
          children: section
        }
      ]
    });
    return doc;
  }
  async toSection() {
    const tokenList = tokenize(this.markdown, this.options);
    if (!this.ignoreImage) {
      const imageList = getImageTokens(tokenList);
      if (imageList.length) {
        await this.downloadImageList(imageList);
      }
    }
    return this.toBlocks(tokenList);
  }
  async downloadImageList(tokens) {
    const imageAdapter = this.options.imageAdapter;
    if (typeof imageAdapter !== "function") {
      throw new Error("MarkdownDocx.imageAdapter is not a function");
    }
    const store = this._imageStore;
    const promises = tokens.map((token) => {
      if (store.has(token.href)) {
        return Promise.resolve(store.get(token.href));
      }
      const cache = {};
      store.set(token.href, cache);
      return imageAdapter(token).then((item) => {
        Object.assign(cache, item);
        return cache;
      });
    });
    return Promise.all(promises);
  }
  toBlocks(tokens, attr = {}) {
    return renderBlocks(this, tokens, attr);
  }
  toTexts(tokens, attr = {}) {
    return renderTokens(this, tokens, attr);
  }
  addFootnote(id, children) {
    this.footnotes[id] = {
      children
    };
  }
  findImage(token) {
    const image = this._imageStore.get(token.href);
    if (!image) {
      return null;
    }
    return image;
  }
};
_MarkdownDocx.defaultOptions = {
  gfm: true
};
var MarkdownDocx = _MarkdownDocx;

// src/index.ts
if (false) {
  const adapters = null;
  MarkdownDocx.defaultOptions.imageAdapter = adapters.downloadImage;
} else {
  const imageAdapter = false ? null.downloadImage : (init_nodejs(), __toCommonJS(nodejs_exports)).downloadImage;
  MarkdownDocx.defaultOptions.imageAdapter = imageAdapter;
}
function markdownDocx(markdown2, options = {}) {
  return MarkdownDocx.covert(markdown2, options);
}
export {
  MarkdownDocx,
  Packer,
  markdownDocx as default,
  markdownDocx,
  styles
};
